@@ -24,11 +24,17 @@ logging.disable(logging.INFO)
 class CLIPVectorizer:
     """ Multimodal vector space for images and texts powered by OpenAI's CLIP """
 
-    def __init__(self, verbose=False):
+    def __init__(self, cpu=None, verbose=False):
         self.verbose = verbose
+        self.device = "cuda" if torch.cuda.is_available() and not cpu else "cpu"
+        if verbose:
+            print("Using device:", self.device)
         TOKENIZER_URL = "https://openaipublic.azureedge.net/clip/bpe_simple_vocab_16e6.txt.gz"
-        MODEL_URL = "https://openaipublic.azureedge.net/clip/models/ \
-            40d365715913c9da98579312b702a82c18be219cc2a73407c4526f58eba950af/ViT-B-32.pt"
+        MODEL_URLS = {
+            "cuda": "https://openaipublic.azureedge.net/clip/models/"
+            + "40d365715913c9da98579312b702a82c18be219cc2a73407c4526f58eba950af/ViT-B-32.pt",
+            "cpu": "https://battle.shawwn.com/sdb/models/ViT-B-32-cpu.pt",
+        }
         db_path = Path.home() / ".config/hypertag/"
         clip_files_path = db_path / "CLIP-files"
         os.makedirs(clip_files_path, exist_ok=True)
@@ -36,12 +42,17 @@ class CLIPVectorizer:
         if not Path(clip_files_path / tokenizer_name).is_file():
             print("Downloading tokenizer...")
             download_url(TOKENIZER_URL, clip_files_path / tokenizer_name)
-        model_name = "model.pt"
+        model_name = self.device + "-model.pt"
         if not Path(clip_files_path / model_name).is_file():
-            print("Downloading CLIP model...")
-            download_url(MODEL_URL, clip_files_path / model_name)
+            print(f"Downloading CLIP {self.device.upper()} model...")
+            download_url(MODEL_URLS[self.device], clip_files_path / model_name)
 
-        self.model = torch.jit.load(str(clip_files_path / model_name)).cuda().eval()
+        self.model = torch.jit.load(str(clip_files_path / model_name), map_location=self.device)
+        self.model = (
+            self.model.eval()
+            if torch.cuda.is_available() and not cpu
+            else self.model.float().eval()
+        )
         self.tokenizer = SimpleTokenizer(bpe_path=str(clip_files_path / tokenizer_name))
         input_resolution = self.model.input_resolution.item()
         self.preprocess = Compose(
@@ -51,8 +62,9 @@ class CLIPVectorizer:
                 ToTensor(),
             ]
         )
-        self.image_mean = torch.tensor([0.48145466, 0.4578275, 0.40821073]).cuda()
-        self.image_std = torch.tensor([0.26862954, 0.26130258, 0.27577711]).cuda()
+
+        self.image_mean = torch.tensor([0.48145466, 0.4578275, 0.40821073]).to(self.device)
+        self.image_std = torch.tensor([0.26862954, 0.26130258, 0.27577711]).to(self.device)
 
         # Build or load index
         corpus_vectors, corpus_paths = self.get_image_corpus()
@@ -65,7 +77,7 @@ class CLIPVectorizer:
             if self.verbose:
                 print("Loading image index...")
             self.index.load_index(str(self.index_path), max_elements=len(corpus_vectors))
-            self.update_index(len(corpus_vectors))
+            self.update_index()
         else:
             # Create the HNSWLIB index
             if not corpus_vectors:
@@ -84,7 +96,7 @@ class CLIPVectorizer:
         # Controlling the recall by setting ef (lower is faster but more inaccuare)
         self.index.set_ef(50)  # ef should always be > top_k_hits
 
-    def update_index(self, current_corpus_length):
+    def update_index(self):
         # Handle new vectorized elements
         with Persistor() as db:
             file_paths = db.get_unindexed_file_paths()
@@ -96,17 +108,19 @@ class CLIPVectorizer:
             new_corpus_vectors.append(json.loads(embedding_vector))
             new_corpus_paths.append(doc_path)
         len_new = len(new_corpus_vectors)
-        len_old = current_corpus_length - 1
+        len_old = self.index.get_current_count()
         new_total_size = len_old + len_new
         if self.verbose:
+            print("CURRENT INDEXED FILES:", len_old)
             print("NEW UNINDEXED FILES:", len_new)
+            print("NEW TOTAL SIZE:", new_total_size)
         if new_corpus_vectors:
             # Add unindexed vectors to index
             self.index.resize_index(new_total_size)
-            print(list(range(len_old, new_total_size)))
+            new_ids = list(range(len_old, new_total_size))
             self.index.add_items(
                 new_corpus_vectors,
-                list(range(len_old, new_total_size)),
+                new_ids,
             )
             if self.verbose:
                 print("Saving updated index to:", self.index_path)
@@ -151,7 +165,7 @@ class CLIPVectorizer:
         # Print results
         results = []
         for corpus_id, score_value in zip(corpus_ids[0], distances[0]):
-            match_file_path = str(corpus_paths[corpus_id])
+            match_file_path = str(corpus_paths[int(corpus_id)])
             file_name = match_file_path.split("/")[-1]
             if path:
                 file_name = match_file_path
@@ -168,7 +182,7 @@ class CLIPVectorizer:
         image = Image.open(path).convert("RGB")
         image = self.preprocess(image)
         image.unsqueeze_(0)
-        image_input = image.cuda()
+        image_input = image.to(self.device)
         image_input -= self.image_mean[:, None, None]
         image_input /= self.image_std[:, None, None]
         with torch.no_grad():
@@ -185,7 +199,7 @@ class CLIPVectorizer:
         for i, tokens in enumerate(text_tokens):
             text_input[i, : len(tokens)] = torch.tensor(tokens)
 
-        text_input = text_input.cuda()
+        text_input = text_input.to(self.device)
 
         with torch.no_grad():
             text_features = self.model.encode_text(text_input).float()
@@ -210,7 +224,7 @@ class TextVectorizer:
             if self.verbose:
                 print("Loading text index...")
             self.index.load_index(str(self.index_path), max_elements=len(corpus_vectors))
-            self.update_index(len(corpus_vectors))
+            self.update_index()
         else:
             # Create the HNSWLIB index
             if not corpus_vectors:
@@ -229,11 +243,11 @@ class TextVectorizer:
         # Controlling the recall by setting ef (lower is faster but more inaccuare)
         self.index.set_ef(50)  # ef should always be > top_k_hits
 
-    def update_index(self, current_corpus_length):
-        # Handle new vectorized but unindexed elements
+    def update_index(self):
+        # Handle new vectorized elements
         with Persistor() as db:
             file_paths = db.get_unindexed_file_paths()
-            compatible_files = [path for path, _file_type in get_text_documents(file_paths)]
+            compatible_files = [p for p, _t in get_text_documents(file_paths)]
             corpus = db.get_file_embedding_vectors(compatible_files)
         new_corpus_paths = []
         new_corpus_vectors = []
@@ -241,17 +255,19 @@ class TextVectorizer:
             new_corpus_vectors.append(json.loads(embedding_vector))
             new_corpus_paths.append(doc_path)
         len_new = len(new_corpus_vectors)
-        len_old = current_corpus_length - 1
+        len_old = self.index.get_current_count()
         new_total_size = len_old + len_new
         if self.verbose:
+            print("CURRENT INDEXED FILES:", len_old)
             print("NEW UNINDEXED FILES:", len_new)
+            print("NEW TOTAL SIZE:", new_total_size)
         if new_corpus_vectors:
             # Add unindexed vectors to index
             self.index.resize_index(new_total_size)
-            print(list(range(len_old, new_total_size)))
+            new_ids = list(range(len_old, new_total_size))
             self.index.add_items(
                 new_corpus_vectors,
-                list(range(len_old, new_total_size)),
+                new_ids,
             )
             if self.verbose:
                 print("Saving updated index to:", self.index_path)
@@ -433,7 +449,7 @@ def clean_transform(text: str, min_words: int, min_word_length: int) -> List[Lis
     for s in sentences:
         ninja_sentence = []
         for w in s.split(" "):
-            if len(w) > 5:
+            if len(w) > 23:  # Words longer than 23 chars are likely missing spaces
                 for nw in wordninja.split(w):  # Split non space seperated word groups up
                     if len(nw) > min_word_length:
                         ninja_sentence.append(nw)
